/* 
 * 
 * Confidential Information of Telekinesys Research Limited (t/a Havok). Not for disclosure or distribution without Havok's
 * prior written consent. This software contains code, techniques and know-how which is confidential and proprietary to Havok.
 * Level 2 and Level 3 source code contains trade secrets of Havok. Havok Software (C) Copyright 1999-2010 Telekinesys Research Limited t/a Havok. All Rights Reserved. Use of this software is subject to the terms of an end user license agreement.
 * 
 */

#include <Demos/demos.h>
#include "HavokTest.h"
#include <Common/Base/Container/LocalArray/hkLocalArray.h>
#include <Animation/Animation/hkaAnimationContainer.h>
#include <Animation/Animation/Playback/Control/Default/hkaDefaultAnimationControl.h>
#include <Animation/Animation/Playback/hkaAnimatedSkeleton.h>
#include <Animation/Animation/Rig/hkaPose.h>
#include <Common/SceneData/Scene/hkxScene.h>
#include <Common/Serialize/Util/hkLoader.h>
#include <Common/Serialize/Util/hkRootLevelContainer.h>
#include <Common/Visualize/hkDebugDisplay.h>
#include <Demos/Animation/Api/MotionExtraction/CharacterController/StateMachine/MyInAirState.h>
#include <Demos/Animation/Api/MotionExtraction/CharacterController/StateMachine/MyStandState.h>
#include <Demos/Animation/Api/MotionExtraction/CharacterController/StateMachine/MyWalkState.h>
#include <Demos/DemoCommon/Utilities/Asset/hkAssetManagementUtil.h>
#include <Demos/DemoCommon/Utilities/GameUtils/GameUtils.h>
#include <Graphics/Bridge/SceneData/hkgSceneDataConverter.h>
#include <Physics/Collide/Shape/Convex/Capsule/hkpCapsuleShape.h>
#include <Physics/Dynamics/Phantom/hkpSimpleShapePhantom.h>
#include <Physics/Utilities/CharacterControl/CharacterProxy/hkpCharacterProxy.h>
#include <Physics/Utilities/Serialize/hkpPhysicsData.h>
#include <Physics/Dynamics/Constraint/Util/hkpConstraintStabilizationUtil.h>

#include "HavokManager.h"
#include "HavokLevel.h"
#include "MakeRigidBody.h"
#include <stdexcept>

HavokTest::HavokTest(hkDemoEnvironment* env)
:	hkDefaultPhysicsDemo(env), m_up(0.0f, 0.0f, 1.0f)
{
	hkVector4 from( 0, -5, 70.0f );
	hkVector4 to  ( 0,0,0 );
	hkVector4 up  ( 0,0,1 );
	setupDefaultCameras( env, from, to, up );

	// Disable warnings: if no renderer									
	if( hkString::strCmp( m_env->m_options->m_renderer, "n" ) == 0 )
	{
		hkError::getInstance().setEnabled(0xf0d1e423, false); //'Could not realize an inplace texture of type PNG.'
	}

	// want to do software skinning always in this demo
	// see HardwareSkinningDemo for how to setup hardware palettes etc
	m_env->m_sceneConverter->setAllowHardwareSkinning(false);


	//
	// Create the World
	//
	{
		{
			hkpWorldCinfo info;
			info.setBroadPhaseWorldSize( 350.0f );
			info.m_gravity.set(0,0,-10);
			info.m_collisionTolerance = 0.1f;
			m_world = new hkpWorld( info );
			m_loader = new hkLoader();
			HavokManager::CreateHavokDemo( m_world, m_loader );
			HavokLevel::CreateInstance();

			m_world->lock();

			hkpAgentRegisterUtil::registerAllAgents(m_world->getCollisionDispatcher());

			setupGraphics();
		}
	
		hkStringBuf assetFile("Resources/MyTest/HavokTest/teapot.hkx"); 

		hkAssetManagementUtil::getFilePath( assetFile );
		
		//HavokLevel::Instance()->AddWorld( assetFile.cString() );
		HavokLevel::Instance()->AddPhysicsSystem( assetFile.cString() );

		//BuildGround( m_world );
		CreateActor();
	}


	m_world->unlock();
}

HavokTest::~HavokTest()
{
	DeleteActor();

	// Re-enable warnings									
	hkError::getInstance().setEnabled(0xf0d1e423, true); 

	m_world->lock();

	if (m_forcedOnTrackball)
		m_env->m_window->getViewport(0)->setNavigationMode(HKG_CAMERA_NAV_FLY);

	HavokLevel::ReleaseInstance();
	HavokManager::ReleaseInstance();

	// need to delete world (delete refs to bodies about to disapear)
	// before we wipe the loader
	m_world->unlock();
	{
		m_world->markForWrite();
		m_world->removeReference();
		m_world = HK_NULL;
	}

	delete m_loader;

}


hkDemo::Result HavokTest::stepDemo()
{
	hkStopwatch* stopwatch = HavokManager::Instance()->GetStopwatch();
	float start = stopwatch->getElapsedSeconds();

	UpdateActor();
	HavokManager::Instance()->Update();

	float end = stopwatch->getElapsedSeconds();

	//OutputDebugPrintf( "stepDemo elapsedSeconds : %f\n", end - start );

	return hkDefaultDemo::stepDemo();
}


void HavokTest::renderPose(const hkaPose& pose) const
{
	/*
	m_world->markForRead();
	// World From Model (accumulated motion) is based on the position of the proxy and the current angle of rotation
	hkQsTransform worldFromModel (m_characterContext.m_characterProxy->getPosition(), hkQuaternion(m_up, m_characterContext.m_currentAngle));
	m_world->unmarkForRead();

	hkTransform graphicsTransform;
	worldFromModel.copyToTransform(graphicsTransform);
	*/
}



static const char helpString[] = \
"This demo shows the Animation system running on a character controlled by the Havok Physics SDK. " \
"The motion of the character proxy (physics) is driven by the animation system.";

HK_DECLARE_DEMO(HavokTest, HK_DEMO_TYPE_ANIMATION | HK_DEMO_TYPE_SERIALIZE, "Havok Test.", helpString);

