//havok_config.cpp
#include <Demos/demos.h>
#include "MakeRigidBody.h"
#include <Common/Base/hkBase.h>
#include <Common/Base/Container/String/hkStringBuf.h>
#include <Common/Base/Ext/hkBaseExt.h>
#include <Common/Base/Memory/System/hkMemorySystem.h>
#include <Common/Base/Memory/System/Util/hkMemoryInitUtil.h>
#include <Common/Base/Memory/Allocator/Malloc/hkMallocAllocator.h>

#include <Physics/Dynamics/World/hkpWorld.h>
#include <Physics/Dynamics/Entity/hkpRigidBody.h>
#include <Physics/Dynamics/Entity/hkpRigidBodyCinfo.h>
#include <Physics/Utilities/Dynamics/Inertia/hkpInertiaTensorComputer.h>
#include <Physics/Collide/Shape/Convex/Box/hkpBoxShape.h>
#include <Physics/Collide/Dispatch/hkpAgentRegisterUtil.h>
#include <Physics/Collide/Filter/Group/hkpGroupFilter.h>
#include <Common/Serialize/Util/hkSerializeDeprecated.h>

#include "HavokManager.h"
#include "HavokActor.h"
#include "HavokCharacter.h"
#include "HavokDefine.h"


const float g_groundHalfHeight = 2.0f;
//ڽ    
const float g_bx = 1.0f, g_by = 2.0f, g_bz = 1.0f;
const int g_nx = 2, g_ny = 2, g_nz = 2;
const int g_NUM_BODIES = g_nx * g_ny * g_nz;
hkpRigidBody *g_bodies[g_NUM_BODIES];

HavokActor* g_actor = NULL;
HavokCharacter* g_character = NULL;


float Angle( float dx, float dy )
{
    float	angle;

	if( !dx )
		dx = float(1e-6);

    angle = atan( dy / dx );

    if( dx < 0 ) 
		angle += HK_REAL_PI;

	//a= a * 180 / HK_REAL_PI;
    return angle;
}


void OutputDebugPrintf(char* fmt, ...) 
{
    char buffer[400];

    memset( buffer, 0, 400 );
    va_list argptr;
    int cnt;

    va_start(argptr, fmt);
    cnt = vsprintf_s(buffer, 400, fmt, argptr);
    va_end(argptr);

    OutputDebugString(buffer);
}


void CreateActor()
{
#ifdef USE_HAVOK_CHARACTER
	g_character = new HavokCharacter;
	//g_character->Create( hkVector4( -10, 0, 0 ) );

	//ship
	g_character->Create( hkVector4( 0, 0, 60 ) );
#else
	g_actor	= new HavokActor;
	//g_actor->CreateCapsuleProxy( 1.7f, 0.4f, hkVector4( -10, 0, 0 ) );

	//ship
	g_actor->CreateCapsuleProxy( hkVector4( 0, 0, 60 ) );
#endif
}

void DeleteActor()
{
#ifdef USE_HAVOK_CHARACTER
	g_character->Destroy();
	delete g_character;
	g_character = NULL;
#else
	g_actor->DeleteProxy();
	delete g_actor;
	g_actor = NULL;
#endif
}

void UpdateActor()
{
#ifdef USE_HAVOK_CHARACTER
	if( g_character )
		g_character->Update( 1.f / 60.f );
#else
	if( g_actor )
		g_actor->UpdateStep( 1.f / 60.f );
#endif
}

//position is center
hkpRigidBody* AddStaticBox( hkpWorld* pWorld , const hkVector4* position, const hkVector4*  width )
{
	const hkVector4& w = *width;
	const hkVector4 halfExtents( w(0)/2.0f, w(1)/2.0f, w(2)/2.0f );
	hkpShape* fixedSurfaceShape = new hkpBoxShape( halfExtents );

	//create rigid body information structure 
	hkpRigidBodyCinfo rigidBodyInfo;
	
	//MOTION_FIXED means static element in game scene
	rigidBodyInfo.m_mass = 0.0;
	rigidBodyInfo.m_shape = fixedSurfaceShape;
	rigidBodyInfo.m_motionType = hkpMotion::MOTION_FIXED;
	rigidBodyInfo.m_position = *position;
	hkVector4 m_fAxis = hkVector4( 0, 0, 1 );
	//hkReal m_fAngle = -0.1f;
	//rigidBodyInfo.m_rotation = hkQuaternion( m_fAxis, m_fAngle);

	//create new rigid body with supplied info
	hkpRigidBody* pRigidBody = new hkpRigidBody( rigidBodyInfo );

	//add rigid body to physics world
	pWorld->lock();
	pWorld->addEntity( pRigidBody );

	//decerase reference counter for rigid body and shape
	pRigidBody->removeReference();
	fixedSurfaceShape->removeReference();

	pWorld->unlock();
	return pRigidBody;
}



hkpRigidBody* BuildGround( hkpWorld* pWorld )
{
	pWorld->markForWrite();

#ifdef USE_HAVOKDEMO
	const hkVector4 halfExtents(20.0f, 20, g_groundHalfHeight );
#else
    const hkVector4 halfExtents(2000.0f, g_groundHalfHeight, 2000.0f);
#endif
    hkpBoxShape* groundShape = new hkpBoxShape(halfExtents);

    hkpRigidBodyCinfo bodyInfo;
    bodyInfo.m_mass = 0.0f; 
    bodyInfo.m_shape = groundShape;
    bodyInfo.m_motionType = hkpMotion::MOTION_FIXED;
#ifdef  USE_HAVOKDEMO
	bodyInfo.m_position.set(0.0f, 0, -g_groundHalfHeight );
#else
	bodyInfo.m_position.set(0.0f, -g_groundHalfHeight, 0.0f);
#endif
	//bodyInfo.m_collisionFilterInfo = HavokManager::LAYER_STATIC_BACKGROUND;

    hkpRigidBody* rigidBody = new hkpRigidBody(bodyInfo);
    groundShape->removeReference();

    pWorld->addEntity( rigidBody );
    rigidBody->removeReference(); 

	pWorld->unmarkForWrite();
	return rigidBody;
}

hkpRigidBody* BuildBox( hkpWorld* pWorld )
{
	pWorld->markForWrite();

    const hkVector4 halfExtents(g_bx, g_by, g_bz);
    hkpShape* shape = new hkpBoxShape(halfExtents, 0.0f);

    hkpMassProperties massProperties;
    hkpInertiaTensorComputer::computeShapeVolumeMassProperties(shape, 5.0f, massProperties);

    hkpRigidBodyCinfo bodyInfo;
    bodyInfo.m_mass = massProperties.m_mass;
    bodyInfo.m_centerOfMass = massProperties.m_centerOfMass;
    bodyInfo.m_inertiaTensor = massProperties.m_inertiaTensor;
    bodyInfo.m_shape = shape;
    bodyInfo.m_motionType = hkpMotion::MOTION_BOX_INERTIA;
	//bodyInfo.m_collisionFilterInfo = HavokManager::LAYER_ACTOR;

    int i = 0;
    for(int x = 0; x < g_nx; x++)
    for(int y = 0; y < g_ny; y++)
    for(int z = 0; z < g_nz; z++) 
	{
        hkpRigidBody *body = new hkpRigidBody(bodyInfo);
		g_bodies[i] = body;

        body->setPosition(hkVector4(x * g_bx * 2, 100.0f + y * g_by * 2, z * g_bz * 2));
        pWorld->addEntity(body);
        body->removeReference();
        i++;
    }
    shape->removeReference();

	pWorld->unmarkForWrite();
	return g_bodies[0];
}
